package com.example.venue_reservation_service.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.example.venue_reservation_service.domain.School;
import com.example.venue_reservation_service.domain.User;
import com.example.venue_reservation_service.entity.PersonInfo;
import com.example.venue_reservation_service.entity.ResponseData;
import com.example.venue_reservation_service.exception.IdentityException;
import com.example.venue_reservation_service.exception.SystemException;
import com.example.venue_reservation_service.mapper.InstitutionMapper;
import com.example.venue_reservation_service.mapper.UserMapper;
import com.example.venue_reservation_service.service.SchoolService;
import com.example.venue_reservation_service.mapper.SchoolMapper;
import com.example.venue_reservation_service.utils.IdentityUtil;
import com.example.venue_reservation_service.vo.Result;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.crypto.NoSuchPaddingException;
import java.io.IOException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.util.List;
import java.util.Optional;

/**
* @author 31586
* @description 针对表【venue_school(学校身份表)】的数据库操作Service实现
* @createDate 2025-01-05 15:42:15
*/
@Service
@Slf4j
public class SchoolServiceImpl extends ServiceImpl<SchoolMapper, School>
    implements SchoolService{

    @Resource
    private InstitutionMapper institutionMapper;

    @Resource
    private UserMapper userMapper;

    private final ObjectMapper mapper = new ObjectMapper();

    @Override
    public Result queryUnits(String type) {
        return Result.
                ok(institutionMapper.selectByType("学生".equalsIgnoreCase(type) ? "学院与教学机构" : null)).
                message("数据获取成功");
    }

    @Override
    public Result userCertify(School school) {
        // 基础用户校验（复用逻辑）
        User user = validateUser(school.getUserId());
        if (user == null) {
            return Result.fail().message("用户信息不存在");
        }

        // 设置用户类型
        user.setType("学生".equals(school.getType()) ? 1 : 2);

        // 教职工身份验证
        if (user.getType() == 2) {
            validateStaffIdentity(school);
        }

        // 更新用户认证状态
        user.setOperableTimes(2);
        user.setIsCertify(1);
        userMapper.updateById(user);

        // 保存学校信息
        save(school);
        return Result.ok(school).message("身份认证成功");
    }

    @Override
    public Result updateCertify(School school) {
        // 基础用户校验（复用逻辑）
        User user = validateUser(school.getUserId());
        if (user == null) {
            return Result.fail().message("用户信息不存在");
        }

        // 校验修改次数
//        if (user.getIsCertify() == 1 && user.getOperableTimes() <= 0) {
//            return Result.fail().message("修改次数已用完，请联系管理员进行重置");
//        }

        // 更新用户认证状态
        user.setOperableTimes(user.getOperableTimes() - 1);
        user.setIsCertify(1);
        userMapper.updateById(user);

        // 更新学校信息
        updateById(school);
        return Result.ok().message("认证信息修改成功");
    }

    /**
     * 公共用户校验逻辑
     */
    private User validateUser(Integer userId) {
        return userMapper.selectById(userId);
    }

    /**
     * 教职工身份验证逻辑
     */
    private void validateStaffIdentity(School school) {
        try {
            // 调用身份认证接口
            String res = IdentityUtil.getGzzxList(school.getSchoolNumber());

            // 结果有效性校验
            if ("-1".equals(res)) {
                throw new IdentityException("身份认证接口调用失败，请检查参数");
            }

            // 解析JSON响应
            ResponseData result = mapper.readValue(res, ResponseData.class);
            if (!"true".equals(result.getSuccess()) || result.getMessage() == null || result.getMessage().isEmpty()) {
                throw new IdentityException("身份信息异常，请稍后重试");
            }

            // 验证返回的教职工信息
            PersonInfo info = result.getMessage().get(0);
            if (!school.getSchoolNumber().equals(info.getEmployeeId()) ||
                    !school.getSchoolNumber().equals(info.getPhone())) {
                throw new IdentityException("教职工信息不匹配");
            }

        } catch (NoSuchAlgorithmException | InvalidKeyException | NoSuchPaddingException e) {
            throw new SystemException("加密组件异常，请联系管理员", e);
        } catch (IOException e) {
            throw new IdentityException("IDENTITY_004", "网络连接失败，请检查网络");
        } catch (Exception e) {
            throw new IdentityException("响应数据解析失败，请稍后重试");
        }
    }
}




